/* vim: set tabstop=8 softtabstop=2 shiftwidth=2 expandtab: */
/*
 *    Copyright (C) 2011 Fabian Deutsch
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

using Media;
using Mdns;

namespace Presence
{
  public class LibPresence : Object
  {
    ApplicationSettings settings;

    LocalRTSPServer rtsp_server;
    MdnsProvider mdns_provider;

    public string rtsp_address {
      get;
      protected set; //FIXME why can it not be private (gbinding)?
    }

    public GLib.HashTable<string,string> peers {
      get;
      private set;
    }

    public signal void peers_discovered (string name, string uri, string txt);
    public signal void peers_lost (string uri);

    public LibPresence.with_settings (ApplicationSettings s)
    {
      this.settings = s;
    }

    public void init (ref unowned string[] args)
    {
        Media.init (ref args);

        init_rtsp_server ();
        init_mdns ();
    }

    void init_mdns ()
    {
        this.mdns_provider = new MdnsProvider ();

        this.settings.bind_property ("mdns-enabled", 
                                     this.mdns_provider, 
                                     "is-published", 
                                     BindingFlags.SYNC_CREATE);
        this.settings.bind_property ("mdns-name", 
                                     this.mdns_provider, 
                                     "name", 
                                     BindingFlags.SYNC_CREATE);
        this.settings.bind_property ("ipv6", 
                                     this.mdns_provider, 
                                     "is-ipv6", 
                                     BindingFlags.SYNC_CREATE);

        this.mdns_provider.service_discovered.connect ( (n,u,t) => {
          this.peers = this.mdns_provider.available_services;
          this.peers_discovered (n, u, t);
        });
        this.mdns_provider.service_lost.connect ( (u) => {
          this.peers = this.mdns_provider.available_services;
          this.peers_lost (u);
        });
    }

    void init_rtsp_server ()
    {
        this.rtsp_server = new LocalRTSPServer ();

        this.settings.notify["rtsp-server-enabled"].connect ( () => {
            if (this.settings.rtsp_server_enabled)
            {
                this.spawn_server_thread ();
            } else {
                rtsp_server.quit ();
            }
        });

        this.settings.bind_property ("rtsp-server-auth-password", 
                                     this.rtsp_server, 
                                     "password", 
                                     BindingFlags.SYNC_CREATE);
        this.settings.bind_property ("ipv6", 
                                     this.rtsp_server, 
                                     "use-ipv6", 
                                     BindingFlags.SYNC_CREATE);
        this.settings.bind_property ("video-scale-factor", 
                                     this.rtsp_server, 
                                     "video-scale-factor", 
                                     BindingFlags.SYNC_CREATE);
/*        this.settings.bind_property ("video-width-request", 
                                     this.rtsp_server, 
                                     "video-width", 
                                     BindingFlags.SYNC_CREATE);
        this.settings.bind_property ("video-height-request", 
                                     this.rtsp_server, 
                                     "video-height", 
                                     BindingFlags.SYNC_CREATE);*/
        this.settings.bind_property ("video-quality-factor", 
                                     this.rtsp_server, 
                                     "video-quality-factor", 
                                     BindingFlags.SYNC_CREATE);
        this.settings.bind_property ("video-fps-factor", 
                                     this.rtsp_server, 
                                     "video-fps-factor", 
                                     BindingFlags.SYNC_CREATE);
        this.settings.bind_property ("video-fps-request", 
                                     this.rtsp_server, 
                                     "video-fps", 
                                     BindingFlags.DEFAULT);
        this.settings.bind_property ("video-device", 
                                     this.rtsp_server, 
                                     "video-device", 
                                     BindingFlags.SYNC_CREATE);

        this.rtsp_server.bind_property ("address", this, "rtsp-address", 
                                        BindingFlags.SYNC_CREATE);
    }

    void spawn_server_thread ()
    {
        Thread<int?>.create<int?> ( () => {
            debug ("Server started.");
            var ctx = new MainContext ();
            var rtsp_server_loop = new MainLoop (ctx);

            rtsp_server.attach (rtsp_server_loop);

            rtsp_server_loop.run ();

            debug ("Server stopped.");
            return 0;
        }, false);
    }
  }
}
